
use tempdb
go

create login bob with password = 'StrongPW1'
create login mary with password = 'StrongPW1'
create login alice with password = 'StrongPW1'
go

create user bob for login bob with default_schema = bobs
create user mary for login mary with default_schema = marys
create user alice for login alice with default_schema = alices
go

create schema bobs authorization bob
go
create schema marys authorization mary
go
create schema alices authorization alice
go

grant create table to bob, mary, alice
grant create procedure to bob, mary, alice

------------------------------------------------------

-- alice creates a table
-- and lets bob read it
execute as user='alice'
create table alices.alicetab (id int)
go
grant select on alices.alicetab to bob
go
revert

------------------------------------------------------

-- bob creates a table
execute as user='bob'
create table bobs.bobtab (id int)
go

-- bob creates a proc to read his table
-- (no ownership chain broken)
create proc bobs.getbob 
as 
select * from bobs.bobtab
go

-- bob creates a proc to read his table with dynamic SQL
-- (ownership chain broken)
create proc bobs.getbobdyn
as
exec('select * from bobs.bobtab')
go

-- bob creates a proc to read alice's table
-- (ownership chain broken)
create proc bobs.getalice
as
select * from alices.alicetab
go

exec bobs.getbob
exec bobs.getbobdyn
exec bobs.getalice

grant execute on getbob to mary
grant execute on getbobdyn to mary
grant execute on getalice to mary
go

revert

-----------------------------------------------------

execute as user='mary'

-- works
exec bobs.getbob
-- doesn't work
exec bobs.getbobdyn
-- doesn't work
exec bobs.getalice

revert

------------------------------------------------------

execute as user='bob'

-- bob creates a proc to read his table with dynamic SQL
-- that executes as him
create proc bobs.getbobdyn2
with execute as owner
as
exec('select * from bobs.bobtab')
select suser_name() 
select original_login()
go

-- bob creates a proc to read alice's table
-- (ownership chain broken)
create proc bobs.getalice2
with execute as owner
as
select * from alices.alicetab
go

exec bobs.getbobdyn2
exec bobs.getalice2

grant execute on bobs.getbobdyn2 to mary
grant execute on bobs.getalice2 to mary

revert

-----------------------------------------------------

execute as user='mary'

-- works now
exec bobs.getbobdyn2
-- works now
exec bobs.getalice2

revert

-----------------------------------------------------

drop login alice
drop login bob
drop login mary
